@@ -1,7 +1,7 @@
-/* $OpenLDAP: pkg/ldap/libraries/libldap/schema.c,v 1.66 2004/09/17 22:15:06 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/libraries/libldap/schema.c,v 1.77 2006/03/31 19:07:53 hyc Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2004 The OpenLDAP Foundation.
+ * Copyright 1998-2006 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -959,26 +959,23 @@ ldap_attributetype2bv(  LDAPAttributeType * at, struct berval *bv )
  * interpretation of the specs).
  */
 
-#define TK_NOENDQUOTE	-2
-#define TK_OUTOFMEM	-1
-#define TK_EOS		0
-#define TK_UNEXPCHAR	1
-#define TK_BAREWORD	2
-#define TK_QDSTRING	3
-#define TK_LEFTPAREN	4
-#define TK_RIGHTPAREN	5
-#define TK_DOLLAR	6
-#define TK_QDESCR	TK_QDSTRING
+typedef enum tk_t {
+	TK_NOENDQUOTE	= -2,
+	TK_OUTOFMEM	= -1,
+	TK_EOS		= 0,
+	TK_UNEXPCHAR	= 1,
+	TK_BAREWORD	= 2,
+	TK_QDSTRING	= 3,
+	TK_LEFTPAREN	= 4,
+	TK_RIGHTPAREN	= 5,
+	TK_DOLLAR	= 6,
+	TK_QDESCR	= TK_QDSTRING
+} tk_t;
 
-struct token {
-	int type;
-	char *sval;
-};
-
-static int
+static tk_t
 get_token( const char ** sp, char ** token_val )
 {
-	int kind;
+	tk_t kind;
 	const char * p;
 	const char * q;
 	char * res;
@@ -1030,6 +1027,9 @@ get_token( const char ** sp, char ** token_val )
 			**sp != ')' &&
 			**sp != '$' &&
 			**sp != '\'' &&
+			/* for suggested minimum upper bound on the number
+			 * of characters <draft-ietf-ldapbis-syntaxes> */
+			**sp != '{' &&
 			**sp != '\0' )
 			(*sp)++;
 		q = *sp;
@@ -1150,7 +1150,7 @@ parse_qdescrs(const char **sp, int *code)
 {
 	char ** res;
 	char ** res1;
-	int kind;
+	tk_t kind;
 	char * sval;
 	int size;
 	int pos;
@@ -1217,7 +1217,7 @@ static char *
 parse_woid(const char **sp, int *code)
 {
 	char * sval;
-	int kind;
+	tk_t kind;
 
 	parse_whsp(sp);
 	kind = get_token(sp, &sval);
@@ -1232,10 +1232,13 @@ parse_woid(const char **sp, int *code)
 
 /* Parse a noidlen */
 static char *
-parse_noidlen(const char **sp, int *code, int *len, int allow_quoted)
+parse_noidlen(const char **sp, int *code, int *len, int flags)
 {
 	char * sval;
+	const char *savepos;
 	int quoted = 0;
+	int allow_quoted = ( flags & LDAP_SCHEMA_ALLOW_QUOTED );
+	int allow_oidmacro = ( flags & LDAP_SCHEMA_ALLOW_OID_MACRO );
 
 	*len = 0;
 	/* Netscape puts the SYNTAX value in quotes (incorrectly) */
@@ -1243,9 +1246,22 @@ parse_noidlen(const char **sp, int *code, int *len, int allow_quoted)
 		quoted = 1;
 		(*sp)++;
 	}
+	savepos = *sp;
 	sval = ldap_int_parse_numericoid(sp, code, 0);
 	if ( !sval ) {
-		return NULL;
+		if ( allow_oidmacro
+			&& *sp == savepos
+			&& *code == LDAP_SCHERR_NODIGIT )
+		{
+			if ( get_token(sp, &sval) != TK_BAREWORD ) {
+				if ( sval != NULL ) {
+					LDAP_FREE(sval);
+				}
+				return NULL;
+			}
+		} else {
+			return NULL;
+		}
 	}
 	if ( **sp == '{' /*}*/ ) {
 		(*sp)++;
@@ -1284,7 +1300,7 @@ parse_oids(const char **sp, int *code, const int allow_quoted)
 {
 	char ** res;
 	char ** res1;
-	int kind;
+	tk_t kind;
 	char * sval;
 	int size;
 	int pos;
@@ -1312,6 +1328,11 @@ parse_oids(const char **sp, int *code, const int allow_quoted)
 		     ( allow_quoted && kind == TK_QDSTRING ) ) {
 			res[pos++] = sval;
 			res[pos] = NULL;
+		} else if ( kind == TK_RIGHTPAREN ) {
+			/* FIXME: be liberal in what we accept... */
+			parse_whsp(sp);
+			LDAP_FREE(res);
+			return NULL;
 		} else {
 			*code = LDAP_SCHERR_UNEXPTOKEN;
 			LDAP_FREE(sval);
@@ -1393,16 +1414,20 @@ add_extension(LDAPSchemaExtensionItem ***extensions,
 	if ( !*extensions ) {
 		*extensions =
 		  LDAP_CALLOC(2, sizeof(LDAPSchemaExtensionItem *));
-		if ( !*extensions )
-		  return 1;
+		if ( !*extensions ) {
+			LDAP_FREE( ext );
+			return 1;
+		}
 		n = 0;
 	} else {
 		for ( n=0; (*extensions)[n] != NULL; n++ )
 	  		;
 		tmp = LDAP_REALLOC(*extensions,
 				   (n+2)*sizeof(LDAPSchemaExtensionItem *));
-		if ( !tmp )
+		if ( !tmp ) {
+			LDAP_FREE( ext );
 			return 1;
+		}
 		*extensions = tmp;
 	}
 	(*extensions)[n] = ext;
@@ -1441,7 +1466,7 @@ ldap_str2syntax( LDAP_CONST char * s,
 	LDAP_CONST char ** errp,
 	LDAP_CONST unsigned flags )
 {
-	int kind;
+	tk_t kind;
 	const char * ss = s;
 	char * sval;
 	int seen_name = 0;
@@ -1583,7 +1608,7 @@ ldap_str2matchingrule( LDAP_CONST char * s,
 	LDAP_CONST char ** errp,
 	LDAP_CONST unsigned flags )
 {
-	int kind;
+	tk_t kind;
 	const char * ss = s;
 	char * sval;
 	int seen_name = 0;
@@ -1782,7 +1807,7 @@ ldap_str2matchingruleuse( LDAP_CONST char * s,
 	LDAP_CONST char ** errp,
 	LDAP_CONST unsigned flags )
 {
-	int kind;
+	tk_t kind;
 	const char * ss = s;
 	char * sval;
 	int seen_name = 0;
@@ -1924,7 +1949,7 @@ ldap_str2matchingruleuse( LDAP_CONST char * s,
 				mru->mru_applies_oids = parse_oids(&ss,
 							     code,
 							     flags);
-				if ( !mru->mru_applies_oids ) {
+				if ( !mru->mru_applies_oids && *code != LDAP_SUCCESS ) {
 					*errp = ss;
 					ldap_matchingruleuse_free(mru);
 					return NULL;
@@ -1984,7 +2009,7 @@ ldap_str2attributetype( LDAP_CONST char * s,
 	LDAP_CONST char ** errp,
 	LDAP_CONST unsigned flags )
 {
-	int kind;
+	tk_t kind;
 	const char * ss = s;
 	char * sval;
 	int seen_name = 0;
@@ -2035,7 +2060,8 @@ ldap_str2attributetype( LDAP_CONST char * s,
 	if ( !at->at_oid ) {
 		if ( ( flags & ( LDAP_SCHEMA_ALLOW_NO_OID
 				| LDAP_SCHEMA_ALLOW_OID_MACRO ) )
-			    && (ss == savepos) ) {
+			    && (ss == savepos) )
+		{
 			/* Backtracking */
 			ss = savepos;
 			kind = get_token(&ss,&sval);
@@ -2052,11 +2078,13 @@ ldap_str2attributetype( LDAP_CONST char * s,
 				     !strcasecmp(sval, "COLLECTIVE") ||
 				     !strcasecmp(sval, "NO-USER-MODIFICATION") ||
 				     !strcasecmp(sval, "USAGE") ||
-				     !strncasecmp(sval, "X-", 2) ) {
+				     !strncasecmp(sval, "X-", 2) )
+				{
 					/* Missing OID, backtrack */
 					ss = savepos;
 				} else if ( flags
-					& LDAP_SCHEMA_ALLOW_OID_MACRO) {
+					& LDAP_SCHEMA_ALLOW_OID_MACRO)
+				{
 					/* Non-numerical OID ... */
 					int len = ss-savepos;
 					at->at_oid = LDAP_MALLOC(len+1);
@@ -2362,7 +2390,7 @@ ldap_str2objectclass( LDAP_CONST char * s,
 	LDAP_CONST char ** errp,
 	LDAP_CONST unsigned flags )
 {
-	int kind;
+	tk_t kind;
 	const char * ss = s;
 	char * sval;
 	int seen_name = 0;
@@ -2437,6 +2465,7 @@ ldap_str2objectclass( LDAP_CONST char * s,
 				}
 			}
 			LDAP_FREE(sval);
+			*code = 0;
 		} else {
 			*errp = ss;
 			ldap_objectclass_free(oc);
@@ -2520,11 +2549,12 @@ ldap_str2objectclass( LDAP_CONST char * s,
 				oc->oc_sup_oids = parse_oids(&ss,
 							     code,
 							     flags);
-				if ( !oc->oc_sup_oids ) {
+				if ( !oc->oc_sup_oids && *code != LDAP_SUCCESS ) {
 					*errp = ss;
 					ldap_objectclass_free(oc);
 					return NULL;
 				}
+				*code = 0;
 			} else if ( !strcasecmp(sval,"ABSTRACT") ) {
 				LDAP_FREE(sval);
 				if ( seen_kind ) {
@@ -2568,11 +2598,12 @@ ldap_str2objectclass( LDAP_CONST char * s,
 				}
 				seen_must = 1;
 				oc->oc_at_oids_must = parse_oids(&ss,code,0);
-				if ( !oc->oc_at_oids_must ) {
+				if ( !oc->oc_at_oids_must && *code != LDAP_SUCCESS ) {
 					*errp = ss;
 					ldap_objectclass_free(oc);
 					return NULL;
 				}
+				*code = 0;
 				parse_whsp(&ss);
 			} else if ( !strcasecmp(sval,"MAY") ) {
 				LDAP_FREE(sval);
@@ -2584,15 +2615,17 @@ ldap_str2objectclass( LDAP_CONST char * s,
 				}
 				seen_may = 1;
 				oc->oc_at_oids_may = parse_oids(&ss,code,0);
-				if ( !oc->oc_at_oids_may ) {
+				if ( !oc->oc_at_oids_may && *code != LDAP_SUCCESS ) {
 					*errp = ss;
 					ldap_objectclass_free(oc);
 					return NULL;
 				}
+				*code = 0;
 				parse_whsp(&ss);
 			} else if ( sval[0] == 'X' && sval[1] == '-' ) {
 				/* Should be parse_qdstrings */
 				ext_vals = parse_qdescrs(&ss, code);
+				*code = 0;
 				if ( !ext_vals ) {
 					*errp = ss;
 					ldap_objectclass_free(oc);
@@ -2644,7 +2677,7 @@ ldap_str2contentrule( LDAP_CONST char * s,
 	LDAP_CONST char ** errp,
 	LDAP_CONST unsigned flags )
 {
-	int kind;
+	tk_t kind;
 	const char * ss = s;
 	char * sval;
 	int seen_name = 0;
@@ -2809,7 +2842,7 @@ ldap_str2contentrule( LDAP_CONST char * s,
 				}
 				seen_must = 1;
 				cr->cr_at_oids_must = parse_oids(&ss,code,0);
-				if ( !cr->cr_at_oids_must ) {
+				if ( !cr->cr_at_oids_must && *code != LDAP_SUCCESS ) {
 					*errp = ss;
 					ldap_contentrule_free(cr);
 					return NULL;
@@ -2825,7 +2858,7 @@ ldap_str2contentrule( LDAP_CONST char * s,
 				}
 				seen_may = 1;
 				cr->cr_at_oids_may = parse_oids(&ss,code,0);
-				if ( !cr->cr_at_oids_may ) {
+				if ( !cr->cr_at_oids_may && *code != LDAP_SUCCESS ) {
 					*errp = ss;
 					ldap_contentrule_free(cr);
 					return NULL;
@@ -2841,7 +2874,7 @@ ldap_str2contentrule( LDAP_CONST char * s,
 				}
 				seen_not = 1;
 				cr->cr_at_oids_not = parse_oids(&ss,code,0);
-				if ( !cr->cr_at_oids_not ) {
+				if ( !cr->cr_at_oids_not && *code != LDAP_SUCCESS ) {
 					*errp = ss;
 					ldap_contentrule_free(cr);
 					return NULL;
@@ -2898,7 +2931,8 @@ ldap_str2structurerule( LDAP_CONST char * s,
 	LDAP_CONST char ** errp,
 	LDAP_CONST unsigned flags )
 {
-	int kind, ret;
+	tk_t kind;
+	int ret;
 	const char * ss = s;
 	char * sval;
 	int seen_name = 0;
@@ -3081,7 +3115,7 @@ ldap_str2nameform( LDAP_CONST char * s,
 	LDAP_CONST char ** errp,
 	LDAP_CONST unsigned flags )
 {
-	int kind;
+	tk_t kind;
 	const char * ss = s;
 	char * sval;
 	int seen_name = 0;
@@ -3201,6 +3235,21 @@ ldap_str2nameform( LDAP_CONST char * s,
 				seen_obsolete = 1;
 				nf->nf_obsolete = LDAP_SCHEMA_YES;
 				parse_whsp(&ss);
+			} else if ( !strcasecmp(sval,"OC") ) {
+				LDAP_FREE(sval);
+				if ( seen_class ) {
+					*code = LDAP_SCHERR_DUPOPT;
+					*errp = ss;
+					ldap_nameform_free(nf);
+					return(NULL);
+				}
+				seen_class = 1;
+				nf->nf_objectclass = parse_woid(&ss,code);
+				if ( !nf->nf_objectclass ) {
+					*errp = ss;
+					ldap_nameform_free(nf);
+					return NULL;
+				}
 			} else if ( !strcasecmp(sval,"MUST") ) {
 				LDAP_FREE(sval);
 				if ( seen_must ) {
@@ -3211,7 +3260,7 @@ ldap_str2nameform( LDAP_CONST char * s,
 				}
 				seen_must = 1;
 				nf->nf_at_oids_must = parse_oids(&ss,code,0);
-				if ( !nf->nf_at_oids_must ) {
+				if ( !nf->nf_at_oids_must && *code != LDAP_SUCCESS ) {
 					*errp = ss;
 					ldap_nameform_free(nf);
 					return NULL;
@@ -3227,7 +3276,7 @@ ldap_str2nameform( LDAP_CONST char * s,
 				}
 				seen_may = 1;
 				nf->nf_at_oids_may = parse_oids(&ss,code,0);
-				if ( !nf->nf_at_oids_may ) {
+				if ( !nf->nf_at_oids_may && *code != LDAP_SUCCESS ) {
 					*errp = ss;
 					ldap_nameform_free(nf);
 					return NULL;
