@@ -1,7 +1,7 @@
-/* $OpenLDAP: pkg/ldap/libraries/libldap/test.c,v 1.50 2004/03/18 00:06:40 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/libraries/libldap/test.c,v 1.55 2006/03/31 18:36:53 hyc Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2004 The OpenLDAP Foundation.
+ * Copyright 1998-2006 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -59,7 +59,7 @@ static char *dnsuffix;
 static char *
 get_line( char *line, int len, FILE *fp, const char *prompt )
 {
-	printf(prompt);
+	fputs(prompt, stdout);
 
 	if ( fgets( line, len, fp ) == NULL )
 		return( NULL );
@@ -169,7 +169,7 @@ get_modlist(
 {
 	static char	buf[256];
 	int		num;
-	LDAPMod		tmp;
+	LDAPMod		tmp = { 0 };
 	LDAPMod		**result;
 	struct berval	**bvals;
 
@@ -205,6 +205,10 @@ get_modlist(
 				    6 ) == 0 ) {
 					if ( file_read( tmp.mod_values[i] + 6,
 					    bvals[i] ) < 0 ) {
+						free( bvals );
+						for ( i = 0; i<num; i++ )
+							free( result[ i ] );
+						free( result );
 						return( NULL );
 					}
 				} else {
