@@ -1,7 +1,7 @@
-/* $OpenLDAP: pkg/ldap/libraries/liblutil/memcmp.c,v 1.6.2.1 2005/01/20 18:03:52 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/libraries/liblutil/memcmp.c,v 1.9 2006/01/03 22:12:11 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2005 The OpenLDAP Foundation.
+ * Copyright 1998-2006 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -21,14 +21,13 @@
  * Memory Compare
  */
 int
-(memcmp)(const void *v1, const void *v2, int n) 
+(lutil_memcmp)(const void *v1, const void *v2, size_t n) 
 {
     if (n != 0) {
 		const unsigned char *s1=v1, *s2=v2;
         do {
-            if (*s1++ != *s2++)
-                return (*--s1 - *--s2);
+            if (*s1++ != *s2++) return *--s1 - *--s2;
         } while (--n != 0);
     }
-    return (0);
+    return 0;
 } 
