@@ -1,5 +1,5 @@
 /* distproc.c - implement distributed procedures */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-ldap/distproc.c,v 1.3 2006/04/02 21:47:13 hallvard Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-ldap/distproc.c,v 1.3.2.2 2006/08/18 04:30:12 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
  * Copyright 2005-2006 The OpenLDAP Foundation.
@@ -22,14 +22,15 @@
 
 #include "portable.h"
 
-#ifdef LDAP_DEVEL
-
 #include <stdio.h>
 
 #include <ac/string.h>
 #include <ac/socket.h>
 
 #include "slap.h"
+
+#ifdef SLAP_DISTPROC
+
 #include "back-ldap.h"
 
 #include "config.h"
@@ -336,9 +337,10 @@ static ConfigTable distproc_cfg[] = {
 			"SYNTAX OMsDirectoryString SINGLE-VALUE )", NULL, NULL },
 	{ "distproc-cache-uri", "TRUE/FALSE",
 		2, 2, 0, ARG_MAGIC|ARG_ON_OFF|DP_CACHE_URI, distproc_cfgen,
-		"( OLcfgOvAt:3.2 NAME 'olcCacheURI' "
+		"( OLcfgOvAt:3.2 NAME 'olcChainCacheURI' "
 			"DESC 'Enables caching of URIs not present in configuration' "
-			"SYNTAX OMsBoolean SINGLE-VALUE )", NULL, NULL },
+			"SYNTAX OMsBoolean "
+			"SINGLE-VALUE )", NULL, NULL },
 	{ NULL, NULL, 0, 0, 0, ARG_IGNORED }
 };
 
@@ -349,7 +351,7 @@ static ConfigOCs distproc_ocs[] = {
 		"SUP olcOverlayConfig "
 		"MAY ( "
 			"olcChainingBehavior $ "
-			"olcCacheURI "
+			"olcChainCacheURI "
 			") )",
 		Cft_Overlay, distproc_cfg, NULL, distproc_cfadd },
 	{ "( OLcfgOvOc:7.2 "
@@ -956,6 +958,7 @@ distproc_initialize( void )
 		return rc;
 	}
 
+#ifdef LDAP_DEVEL
 	rc = supported_feature_load( &slap_FEATURE_CANCHAINOPS );
 	if ( rc != LDAP_SUCCESS ) {
 		Debug( LDAP_DEBUG_ANY, "slapd-distproc: "
@@ -963,6 +966,7 @@ distproc_initialize( void )
 			rc, 0, 0 );
 		return rc;
 	}
+#endif
 
 	rc = register_supported_control( LDAP_CONTROL_X_RETURNCONTREF,
 			SLAP_CTRL_GLOBAL|SLAP_CTRL_ACCESS|SLAP_CTRL_HIDE, NULL,
@@ -1006,4 +1010,4 @@ distproc_initialize( void )
 	return overlay_register( &distproc );
 }
 
-#endif /* LDAP_DEVEL */
+#endif /* SLAP_DISTPROC */
