@@ -1,8 +1,8 @@
 /* log.c - deal with log subsystem */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-monitor/log.c,v 1.45.2.1 2005/01/20 18:04:00 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-monitor/log.c,v 1.56 2006/01/03 22:12:21 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2001-2005 The OpenLDAP Foundation.
+ * Copyright 2001-2006 The OpenLDAP Foundation.
  * Portions Copyright 2001-2003 Pierangelo Masarati.
  * All rights reserved.
  *
@@ -31,39 +31,25 @@
 #include "ldif.h"
 #include "back-monitor.h"
 
+static int 
+monitor_subsys_log_open( 
+	BackendDB		*be,
+	monitor_subsys_t	*ms );
+
+static int 
+monitor_subsys_log_modify( 
+	Operation		*op,
+	SlapReply		*rs,
+	Entry 			*e );
+
 /*
  * log mutex
  */
 ldap_pvt_thread_mutex_t		monitor_log_mutex;
 
-static struct {
-	int i;
-	struct berval s;
-	struct berval n;
-} int_2_level[] = {
-	{ LDAP_DEBUG_TRACE,	BER_BVC("Trace"),	BER_BVNULL },
-	{ LDAP_DEBUG_PACKETS,	BER_BVC("Packets"),	BER_BVNULL },
-	{ LDAP_DEBUG_ARGS,	BER_BVC("Args"),	BER_BVNULL },
-	{ LDAP_DEBUG_CONNS,	BER_BVC("Conns"),	BER_BVNULL },
-	{ LDAP_DEBUG_BER,	BER_BVC("BER"),	BER_BVNULL },
-	{ LDAP_DEBUG_FILTER,	BER_BVC("Filter"),	BER_BVNULL },
-	{ LDAP_DEBUG_CONFIG,	BER_BVC("Config"),	BER_BVNULL },	/* useless */
-	{ LDAP_DEBUG_ACL,	BER_BVC("ACL"),	BER_BVNULL },
-	{ LDAP_DEBUG_STATS,	BER_BVC("Stats"),	BER_BVNULL },
-	{ LDAP_DEBUG_STATS2,	BER_BVC("Stats2"),	BER_BVNULL },
-	{ LDAP_DEBUG_SHELL,	BER_BVC("Shell"),	BER_BVNULL },
-	{ LDAP_DEBUG_PARSE,	BER_BVC("Parse"),	BER_BVNULL },
-	{ LDAP_DEBUG_CACHE,	BER_BVC("Cache"),	BER_BVNULL },
-	{ LDAP_DEBUG_INDEX,	BER_BVC("Index"),	BER_BVNULL },
-	{ 0,			BER_BVNULL,	BER_BVNULL }
-};
-
-static int loglevel2int( struct berval *l );
-static int int2loglevel( int n );
-
-static int add_values( Entry *e, Modification *mod, int *newlevel );
-static int delete_values( Entry *e, Modification *mod, int *newlevel );
-static int replace_values( Entry *e, Modification *mod, int *newlevel );
+static int add_values( Operation *op, Entry *e, Modification *mod, int *newlevel );
+static int delete_values( Operation *op, Entry *e, Modification *mod, int *newlevel );
+static int replace_values( Operation *op, Entry *e, Modification *mod, int *newlevel );
 
 /*
  * initializes log subentry
@@ -71,66 +57,55 @@ static int replace_values( Entry *e, Modification *mod, int *newlevel );
 int
 monitor_subsys_log_init(
 	BackendDB		*be,
-	monitor_subsys_t	*ms
-)
+	monitor_subsys_t	*ms )
 {
-	monitor_info_t	*mi;
-	Entry		*e;
-	int		i;
-	struct berval	desc[] = {
-		BER_BVC("This entry allows to set the log level runtime."),
-		BER_BVC("Set the attribute 'managedInfo' to the desired log levels."),
-		BER_BVNULL
-	};
+	ms->mss_open = monitor_subsys_log_open;
+	ms->mss_modify = monitor_subsys_log_modify;
 
 	ldap_pvt_thread_mutex_init( &monitor_log_mutex );
 
-	mi = ( monitor_info_t * )be->be_private;
-
-	if ( monitor_cache_get( mi, &ms->mss_ndn, 
-				&e ) ) {
-		Debug( LDAP_DEBUG_ANY,
-			"monitor_subsys_log_init: "
-			"unable to get entry \"%s\"\n",
-			ms->mss_ndn.bv_val, 0, 0 );
-		return( -1 );
-	}
-
-	/* initialize the debug level(s) */
-	for ( i = 0; int_2_level[ i ].i != 0; i++ ) {
-		if ( mi->mi_ad_managedInfo->ad_type->sat_equality->smr_normalize ) {
-			int	rc;
-
-			rc = (*mi->mi_ad_managedInfo->ad_type->sat_equality->smr_normalize)(
-					SLAP_MR_VALUE_OF_ATTRIBUTE_SYNTAX,
-					mi->mi_ad_managedInfo->ad_type->sat_syntax,
-					mi->mi_ad_managedInfo->ad_type->sat_equality,
-					&int_2_level[ i ].s,
-					&int_2_level[ i ].n, NULL );
-			if ( rc ) {
-				return( -1 );
-			}
-		}
-
-		if ( int_2_level[ i ].i & ldap_syslog ) {
-			attr_merge_one( e, mi->mi_ad_managedInfo,
-					&int_2_level[ i ].s,
-					&int_2_level[ i ].n );
-		}
-	}
-
-	attr_merge( e, mi->mi_ad_description, desc, NULL );
-
-	monitor_cache_release( mi, e );
-
 	return( 0 );
 }
 
-int 
+/*
+ * opens log subentry
+ */
+int
+monitor_subsys_log_open(
+	BackendDB		*be,
+	monitor_subsys_t	*ms )
+{
+	BerVarray	bva = NULL;
+
+	if ( loglevel2bvarray( ldap_syslog, &bva ) == 0 && bva != NULL ) {
+		monitor_info_t	*mi;
+		Entry		*e;
+
+		mi = ( monitor_info_t * )be->be_private;
+
+		if ( monitor_cache_get( mi, &ms->mss_ndn, &e ) ) {
+			Debug( LDAP_DEBUG_ANY,
+				"monitor_subsys_log_init: "
+				"unable to get entry \"%s\"\n",
+				ms->mss_ndn.bv_val, 0, 0 );
+			ber_bvarray_free( bva );
+			return( -1 );
+		}
+
+		attr_merge_normalize( e, mi->mi_ad_managedInfo, bva, NULL );
+		ber_bvarray_free( bva );
+
+		monitor_cache_release( mi, e );
+	}
+
+	return( 0 );
+}
+
+static int 
 monitor_subsys_log_modify( 
 	Operation		*op,
-	Entry 			*e
-)
+	SlapReply		*rs,
+	Entry 			*e )
 {
 	monitor_info_t	*mi = ( monitor_info_t * )op->o_bd->be_private;
 	int		rc = LDAP_OTHER;
@@ -154,32 +129,32 @@ monitor_subsys_log_modify(
 		 */
 		if ( is_at_operational( mod->sm_desc->ad_type ) ) {
 			( void ) attr_delete( &e->e_attrs, mod->sm_desc );
-			rc = attr_merge( e, mod->sm_desc, mod->sm_values, mod->sm_nvalues );
-			if ( rc != 0 ) {
-				rc = LDAP_OTHER;
+			rc = rs->sr_err = attr_merge( e, mod->sm_desc,
+					mod->sm_values, mod->sm_nvalues );
+			if ( rc != LDAP_SUCCESS ) {
 				break;
 			}
 			continue;
 
 		/*
-		 * only the monitor description attribute can be modified
+		 * only the "managedInfo" attribute can be modified
 		 */
-		} else if ( mod->sm_desc != mi->mi_ad_managedInfo) {
-			rc = LDAP_UNWILLING_TO_PERFORM;
+		} else if ( mod->sm_desc != mi->mi_ad_managedInfo ) {
+			rc = rs->sr_err = LDAP_UNWILLING_TO_PERFORM;
 			break;
 		}
 
 		switch ( mod->sm_op ) {
 		case LDAP_MOD_ADD:
-			rc = add_values( e, mod, &newlevel );
+			rc = add_values( op, e, mod, &newlevel );
 			break;
 			
 		case LDAP_MOD_DELETE:
-			rc = delete_values( e, mod, &newlevel );
+			rc = delete_values( op, e, mod, &newlevel );
 			break;
 
 		case LDAP_MOD_REPLACE:
-			rc = replace_values( e, mod, &newlevel );
+			rc = replace_values( op, e, mod, &newlevel );
 			break;
 
 		default:
@@ -188,6 +163,7 @@ monitor_subsys_log_modify(
 		}
 
 		if ( rc != LDAP_SUCCESS ) {
+			rs->sr_err = rc;
 			break;
 		}
 	}
@@ -199,15 +175,16 @@ monitor_subsys_log_modify(
 
 		/* check for abandon */
 		if ( op->o_abandon ) {
-			rc = SLAPD_ABANDON;
+			rc = rs->sr_err = SLAPD_ABANDON;
 
 			goto cleanup;
 		}
 
 		/* check that the entry still obeys the schema */
-		rc = entry_schema_check( be_monitor, e, save_attrs, 
-				&text, textbuf, sizeof( textbuf ) );
+		rc = entry_schema_check( op, e, save_attrs, 0,
+			&text, textbuf, sizeof( textbuf ) );
 		if ( rc != LDAP_SUCCESS ) {
+			rs->sr_err = rc;
 			goto cleanup;
 		}
 
@@ -236,39 +213,11 @@ cleanup:;
 	
 	ldap_pvt_thread_mutex_unlock( &monitor_log_mutex );
 
-	return( rc );
-}
-
-static int
-loglevel2int( struct berval *l )
-{
-	int		i;
-	
-	for ( i = 0; int_2_level[ i ].i != 0; i++ ) {
-		if ( l->bv_len != int_2_level[ i ].s.bv_len ) {
-			continue;
-		}
-
-		if ( strcasecmp( l->bv_val, int_2_level[ i ].s.bv_val ) == 0 ) {
-			return int_2_level[ i ].i;
-		}
+	if ( rc == LDAP_SUCCESS ) {
+		rc = SLAP_CB_CONTINUE;
 	}
 
-	return 0;
-}
-
-static int
-int2loglevel( int n )
-{
-	int		i;
-	
-	for ( i = 0; int_2_level[ i ].i != 0; i++ ) {
-		if ( int_2_level[ i ].i == n ) {
-			return i;
-		}
-	}
-
-	return -1;
+	return rc;
 }
 
 static int
@@ -276,28 +225,27 @@ check_constraints( Modification *mod, int *newlevel )
 {
 	int		i;
 
-	for ( i = 0; mod->sm_values && !BER_BVISNULL( &mod->sm_values[ i ] ); i++ ) {
+	if ( mod->sm_nvalues != NULL ) {
+		ber_bvarray_free( mod->sm_nvalues );
+		mod->sm_nvalues = NULL;
+	}
+
+	for ( i = 0; !BER_BVISNULL( &mod->sm_values[ i ] ); i++ ) {
 		int		l;
-		
-		l = loglevel2int( &mod->sm_values[ i ] );
-		if ( !l ) {
+		struct berval	bv;
+
+		if ( str2loglevel( mod->sm_values[ i ].bv_val, &l ) ) {
 			return LDAP_CONSTRAINT_VIOLATION;
 		}
 
-		if ( ( l = int2loglevel( l ) ) == -1 ) {
-			return LDAP_OTHER;
+		if ( loglevel2bv( l, &bv ) ) {
+			return LDAP_CONSTRAINT_VIOLATION;
 		}
-
-		assert( int_2_level[ l ].s.bv_len
-				== mod->sm_values[ i ].bv_len );
+		
+		assert( bv.bv_len == mod->sm_values[ i ].bv_len );
 		
 		AC_MEMCPY( mod->sm_values[ i ].bv_val,
-				int_2_level[ l ].s.bv_val,
-				int_2_level[ l ].s.bv_len );
-
-		AC_MEMCPY( mod->sm_nvalues[ i ].bv_val,
-				int_2_level[ l ].n.bv_val,
-				int_2_level[ l ].n.bv_len );
+				bv.bv_val, bv.bv_len );
 
 		*newlevel |= l;
 	}
@@ -306,12 +254,14 @@ check_constraints( Modification *mod, int *newlevel )
 }	
 
 static int 
-add_values( Entry *e, Modification *mod, int *newlevel )
+add_values( Operation *op, Entry *e, Modification *mod, int *newlevel )
 {
 	Attribute	*a;
 	int		i, rc;
 	MatchingRule 	*mr = mod->sm_desc->ad_type->sat_equality;
 
+	assert( mod->sm_values != NULL );
+
 	rc = check_constraints( mod, newlevel );
 	if ( rc != LDAP_SUCCESS ) {
 		return rc;
@@ -320,7 +270,7 @@ add_values( Entry *e, Modification *mod, int *newlevel )
 	a = attr_find( e->e_attrs, mod->sm_desc );
 
 	if ( a != NULL ) {
-		/* "description" SHOULD have appropriate rules ... */
+		/* "managedInfo" SHOULD have appropriate rules ... */
 		if ( mr == NULL || !mr->smr_match ) {
 			return LDAP_INAPPROPRIATE_MATCHING;
 		}
@@ -333,7 +283,8 @@ add_values( Entry *e, Modification *mod, int *newlevel )
 
 			rc = asserted_value_validate_normalize(
 				mod->sm_desc, mr, SLAP_MR_EQUALITY,
-				&mod->sm_values[ i ], &asserted, &text, NULL );
+				&mod->sm_values[ i ], &asserted, &text,
+				op->o_tmpmemctx );
 
 			if ( rc != LDAP_SUCCESS ) {
 				return rc;
@@ -342,7 +293,7 @@ add_values( Entry *e, Modification *mod, int *newlevel )
 			for ( j = 0; !BER_BVISNULL( &a->a_vals[ j ] ); j++ ) {
 				int match;
 				int rc = value_match( &match, mod->sm_desc, mr,
-					0, &a->a_vals[ j ], &asserted, &text );
+					0, &a->a_nvals[ j ], &asserted, &text );
 
 				if ( rc == LDAP_SUCCESS && match == 0 ) {
 					free( asserted.bv_val );
@@ -355,9 +306,9 @@ add_values( Entry *e, Modification *mod, int *newlevel )
 	}
 
 	/* no - add them */
-	rc = attr_merge( e, mod->sm_desc, mod->sm_values, mod->sm_nvalues );
+	rc = attr_merge_normalize( e, mod->sm_desc, mod->sm_values,
+		op->o_tmpmemctx );
 	if ( rc != LDAP_SUCCESS ) {
-		/* this should return result of attr_mergeit */
 		return rc;
 	}
 
@@ -365,25 +316,19 @@ add_values( Entry *e, Modification *mod, int *newlevel )
 }
 
 static int
-delete_values( Entry *e, Modification *mod, int *newlevel )
+delete_values( Operation *op, Entry *e, Modification *mod, int *newlevel )
 {
 	int             i, j, k, found, rc, nl = 0;
 	Attribute       *a;
 	MatchingRule 	*mr = mod->sm_desc->ad_type->sat_equality;
 
-	rc = check_constraints( mod, &nl );
-	if ( rc != LDAP_SUCCESS ) {
-		return rc;
-	}
-
-	*newlevel &= ~nl;
-
 	/* delete the entire attribute */
 	if ( mod->sm_values == NULL ) {
 		int rc = attr_delete( &e->e_attrs, mod->sm_desc );
 
 		if ( rc ) {
 			rc = LDAP_NO_SUCH_ATTRIBUTE;
+
 		} else {
 			*newlevel = 0;
 			rc = LDAP_SUCCESS;
@@ -391,6 +336,13 @@ delete_values( Entry *e, Modification *mod, int *newlevel )
 		return rc;
 	}
 
+	rc = check_constraints( mod, &nl );
+	if ( rc != LDAP_SUCCESS ) {
+		return rc;
+	}
+
+	*newlevel &= ~nl;
+
 	if ( mr == NULL || !mr->smr_match ) {
 		/* disallow specific attributes from being deleted if
 		 * no equality rule */
@@ -411,7 +363,8 @@ delete_values( Entry *e, Modification *mod, int *newlevel )
 
 		rc = asserted_value_validate_normalize(
 				mod->sm_desc, mr, SLAP_MR_EQUALITY,
-				&mod->sm_values[ i ], &asserted, &text, NULL );
+				&mod->sm_values[ i ], &asserted, &text,
+				op->o_tmpmemctx );
 
 		if( rc != LDAP_SUCCESS ) return rc;
 
@@ -419,8 +372,7 @@ delete_values( Entry *e, Modification *mod, int *newlevel )
 		for ( j = 0; !BER_BVISNULL( &a->a_vals[ j ] ); j++ ) {
 			int match;
 			int rc = value_match( &match, mod->sm_desc, mr,
-				0,
-				&a->a_vals[ j ], &asserted, &text );
+				0, &a->a_nvals[ j ], &asserted, &text );
 
 			if( rc == LDAP_SUCCESS && match != 0 ) {
 				continue;
@@ -430,6 +382,14 @@ delete_values( Entry *e, Modification *mod, int *newlevel )
 			found = 1;
 
 			/* delete it */
+			if ( a->a_nvals != a->a_vals ) {
+				free( a->a_nvals[ j ].bv_val );
+				for ( k = j + 1; !BER_BVISNULL( &a->a_nvals[ k ] ); k++ ) {
+					a->a_nvals[ k - 1 ] = a->a_nvals[ k ];
+				}
+				BER_BVZERO( &a->a_nvals[ k - 1 ] );
+			}
+
 			free( a->a_vals[ j ].bv_val );
 			for ( k = j + 1; !BER_BVISNULL( &a->a_vals[ k ] ); k++ ) {
 				a->a_vals[ k - 1 ] = a->a_vals[ k ];
@@ -461,14 +421,16 @@ delete_values( Entry *e, Modification *mod, int *newlevel )
 }
 
 static int
-replace_values( Entry *e, Modification *mod, int *newlevel )
+replace_values( Operation *op, Entry *e, Modification *mod, int *newlevel )
 {
 	int rc;
 
-	*newlevel = 0;
-	rc = check_constraints( mod, newlevel );
-	if ( rc != LDAP_SUCCESS ) {
-		return rc;
+	if ( mod->sm_values != NULL ) {
+		*newlevel = 0;
+		rc = check_constraints( mod, newlevel );
+		if ( rc != LDAP_SUCCESS ) {
+			return rc;
+		}
 	}
 
 	rc = attr_delete( &e->e_attrs, mod->sm_desc );
@@ -478,7 +440,8 @@ replace_values( Entry *e, Modification *mod, int *newlevel )
 	}
 
 	if ( mod->sm_values != NULL ) {
-		rc = attr_merge( e, mod->sm_desc, mod->sm_values, mod->sm_nvalues );
+		rc = attr_merge_normalize( e, mod->sm_desc, mod->sm_values,
+				op->o_tmpmemctx );
 		if ( rc != LDAP_SUCCESS ) {
 			return rc;
 		}
