@@ -1,7 +1,7 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-passwd/proto-passwd.h,v 1.2 2004/11/15 18:47:04 ando Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-passwd/proto-passwd.h,v 1.5 2006/01/03 22:12:23 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2004 The OpenLDAP Foundation.
+ * Copyright 1998-2006 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -19,13 +19,13 @@
 LDAP_BEGIN_DECL
 
 extern BI_init		passwd_back_initialize;
-
+extern BI_open		passwd_back_open;
 extern BI_destroy	passwd_back_destroy;
-
 extern BI_db_config	passwd_back_db_config;
-
 extern BI_op_search	passwd_back_search;
 
+extern AttributeDescription	*ad_sn;
+extern AttributeDescription	*ad_desc;
 LDAP_END_DECL
 
 #endif /* PROTO_PASSWD_H */
