@@ -1,4 +1,4 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-sql/entry-id.c,v 1.67 2006/01/03 22:12:24 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-sql/entry-id.c,v 1.67.2.1 2006/08/17 23:50:31 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
  * Copyright 1999-2006 The OpenLDAP Foundation.
@@ -90,7 +90,7 @@ backsql_dn2id(
 {
 	backsql_info		*bi = op->o_bd->be_private;
 	SQLHSTMT		sth = SQL_NULL_HSTMT; 
-	BACKSQL_ROW_NTS		row;
+	BACKSQL_ROW_NTS		row = { 0 };
 	RETCODE 		rc;
 	int			res;
 	struct berval		realndn = BER_BVNULL;
