@@ -1,5 +1,5 @@
 /* backover.c - backend overlay routines */
-/* $OpenLDAP: pkg/ldap/servers/slapd/backover.c,v 1.71.2.1 2006/04/18 17:33:07 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/backover.c,v 1.71.2.3 2006/08/20 17:04:46 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
  * Copyright 2003-2006 The OpenLDAP Foundation.
@@ -210,10 +210,13 @@ over_db_destroy(
 
 	rc = over_db_func( be, db_destroy );
 
-	for (next = on->on_next; on; on=next) {
-		next = on->on_next;
-		free( on );
+	if ( on ) {
+		for (next = on->on_next; on; on=next) {
+			next = on->on_next;
+			free( on );
+		}
 	}
+
 	free( oi );
 	return rc;
 }
@@ -1054,7 +1057,11 @@ overlay_config( BackendDB *be, const char *ov )
 		be->bd_info = (BackendInfo *)on2;
 		rc = on2->on_bi.bi_db_init( be );
 		be->bd_info = (BackendInfo *)oi;
-		if ( rc ) return rc;
+		if ( rc ) {
+			oi->oi_list = on2->on_next;
+			ch_free( on2 );
+			return rc;
+		}
 	}
 
 	return 0;
