@@ -1,4 +1,4 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/connection.c,v 1.358.2.1 2006/05/15 17:04:41 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/connection.c,v 1.358.2.3 2006/08/20 17:04:46 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
  * Copyright 1998-2006 The OpenLDAP Foundation.
@@ -731,22 +731,17 @@ connection_destroy( Connection *c )
 	}
 
 	ber_sockbuf_ctrl( sb, LBER_SB_OPT_GET_FD, &sd );
-	slapd_sd_lock();
-
-	ber_sockbuf_free( sb );
 
 	/* c must be fully reset by this point; when we call slapd_remove
 	 * it may get immediately reused by a new connection.
 	 */
 	if ( sd != AC_SOCKET_INVALID ) {
-		slapd_remove( sd, 1, 0, 1 );
+		slapd_remove( sd, sb, 1, 0, 0 );
 
 		Statslog( LDAP_DEBUG_STATS, (close_reason
 									 ? "conn=%lu fd=%ld closed (%s)\n"
 									 : "conn=%lu fd=%ld closed\n"),
 			connid, (long) sd, close_reason, 0, 0 );
-	} else {
-		slapd_sd_unlock();
 	}
 }
 
@@ -919,6 +914,17 @@ Connection* connection_next( Connection *c, ber_socket_t *index )
 		if( connections[*index].c_struct_state == SLAP_C_USED ) {
 			assert( connections[*index].c_conn_state != SLAP_C_INVALID );
 			c = &connections[(*index)++];
+			if ( ldap_pvt_thread_mutex_trylock( &c->c_mutex )) {
+				/* avoid deadlock */
+				ldap_pvt_thread_mutex_unlock( &connections_mutex );
+				ldap_pvt_thread_mutex_lock( &c->c_mutex );
+				ldap_pvt_thread_mutex_lock( &connections_mutex );
+				if ( c->c_struct_state != SLAP_C_USED ) {
+					ldap_pvt_thread_mutex_unlock( &c->c_mutex );
+					c = NULL;
+					continue;
+				}
+			}
 			break;
 		}
 
@@ -926,7 +932,6 @@ Connection* connection_next( Connection *c, ber_socket_t *index )
 		assert( connections[*index].c_conn_state == SLAP_C_INVALID );
 	}
 
-	if( c != NULL ) ldap_pvt_thread_mutex_lock( &c->c_mutex );
 	ldap_pvt_thread_mutex_unlock( &connections_mutex );
 	return c;
 }
@@ -972,7 +977,7 @@ void connection_done( Connection *c )
 /*
  * NOTE: keep in sync with enum in slapd.h
  */
-static int (*opfun[])( Operation *op, SlapReply *rs ) = {
+static BI_op_func *opfun[] = {
 	do_bind,
 	do_unbind,
 	do_add,
@@ -1208,6 +1213,7 @@ void connection_client_stop(
 	ber_socket_t s )
 {
 	Connection *c;
+	Sockbuf *sb;
 
 	/* get (locked) connection */
 	c = connection_get( s );
@@ -1218,14 +1224,13 @@ void connection_client_stop(
 	c->c_conn_state = SLAP_C_INVALID;
 	c->c_struct_state = SLAP_C_UNUSED;
 	c->c_close_reason = "?";			/* should never be needed */
-	slapd_sd_lock();
-	ber_sockbuf_free( c->c_sb );
-	slapd_remove( s, 0, 1, 1 );
+	sb = c->c_sb;
 	c->c_sb = ber_sockbuf_alloc( );
 	{
 		ber_len_t max = sockbuf_max_incoming;
 		ber_sockbuf_ctrl( c->c_sb, LBER_SB_OPT_SET_MAX_INCOMING, &max );
 	}
+	slapd_remove( s, sb, 0, 1, 0 );
 
 	connection_return( c );
 }
