@@ -1,5 +1,5 @@
 /* limits.c - routines to handle regex-based size and time limits */
-/* $OpenLDAP: pkg/ldap/servers/slapd/limits.c,v 1.73.2.1 2006/04/11 16:07:17 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/limits.c,v 1.73.2.2 2006/08/17 23:50:28 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
  * Copyright 1998-2006 The OpenLDAP Foundation.
@@ -926,8 +926,6 @@ limits_unparse( struct slap_limits *lim, struct berval *bv, ber_len_t buflen )
 			break;
 		}
 	}
-	if ( WHATSLEFT <= STRLENOF( " " ) ) return -1;
-	*ptr++ = ' ';
 	bv->bv_len = ptr - bv->bv_val;
 	btmp.bv_val = ptr;
 	btmp.bv_len = 0;
@@ -947,7 +945,7 @@ limits_unparse_one( struct slap_limits_set *lim, int which, struct berval *bv, b
 {
 	char *ptr;
 
-	if ( !bv || !bv->bv_val ) return;
+	if ( !bv || !bv->bv_val ) return -1;
 
 	ptr = bv->bv_val;
 
@@ -1085,6 +1083,8 @@ t_hard:
 		*ptr = '\0';
 		bv->bv_len = ptr - bv->bv_val;
 	}
+
+	return 0;
 }
 
 int
