@@ -1,8 +1,8 @@
 /* rwm.h - dn rewrite/attribute mapping header file */
-/* $OpenLDAP: pkg/ldap/servers/slapd/overlays/rwm.h,v 1.9.2.1 2005/01/20 18:04:04 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/overlays/rwm.h,v 1.15 2006/01/03 22:12:25 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1999-2005 The OpenLDAP Foundation.
+ * Copyright 1999-2006 The OpenLDAP Foundation.
  * Portions Copyright 1999-2003 Howard Chu.
  * Portions Copyright 2000-2003 Pierangelo Masarati.
  * All rights reserved.
@@ -80,6 +80,11 @@ struct ldaprwmap {
 	 */
 	struct ldapmap rwm_oc;
 	struct ldapmap rwm_at;
+
+#define	RWM_F_NONE			0x0000U
+#define	RWM_F_SUPPORT_T_F		0x4000U
+#define	RWM_F_SUPPORT_T_F_DISCOVER	0x8000U
+	unsigned	rwm_flags;
 };
 
 /* Whatever context ldap_back_dn_massage needs... */
@@ -110,7 +115,7 @@ void rwm_map ( struct ldapmap *map, struct berval *s, struct berval *m,
 	int remap );
 int rwm_mapping ( struct ldapmap *map, struct berval *s,
 		struct ldapmapping **m, int remap );
-#define RWM_MAP	0
+#define RWM_MAP		0
 #define RWM_REMAP	1
 char *
 rwm_map_filter(
@@ -133,6 +138,8 @@ rwm_map_attrnames(
 		AttributeName **anp,
 		int remap );
 
+extern void rwm_mapping_dst_free ( void *mapping );
+
 extern void rwm_mapping_free ( void *mapping );
 
 extern int rwm_map_config(
@@ -145,6 +152,7 @@ extern int rwm_map_config(
 
 extern int
 rwm_filter_map_rewrite(
+		Operation		*op,
 		dncookie		*dc,
 		Filter			*f,
 		struct berval		*fstr );
