@@ -1,4 +1,4 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/slapi/slapi_utils.c,v 1.189 2006/03/01 15:29:31 lukeh Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/slapi/slapi_utils.c,v 1.189.2.1 2006/08/17 23:50:32 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
  * Copyright 2002-2006 The OpenLDAP Foundation.
@@ -1349,8 +1349,13 @@ slapi_send_ldap_result(
 	} else {
 		if ( pb->pb_op->o_tag == LDAP_REQ_SEARCH )
 			rs->sr_nentries = nentries;
+		if ( urls != NULL )
+			bvptr2obj( urls, &rs->sr_ref );
 
 		send_ldap_result( pb->pb_op, rs );
+
+		if ( urls != NULL )
+			slapi_ch_free( (void **)&rs->sr_ref );
 	}
 }
 
@@ -1363,7 +1368,7 @@ slapi_send_ldap_search_entry(
 	int		attrsonly )
 {
 	SlapReply		rs = { REP_SEARCH };
-	int			i = 0;
+	int			i = 0, j = 0;
 	AttributeName		*an = NULL;
 	const char		*text;
 	int			rc;
@@ -1377,19 +1382,17 @@ slapi_send_ldap_search_entry(
 	}
 
 	if ( i ) {
-		an = (AttributeName *) slapi_ch_malloc( (i+1) * sizeof(AttributeName) );
+		an = (AttributeName *) slapi_ch_calloc( i + 1, sizeof(AttributeName) );
 		for ( i = 0; attrs[i] != NULL; i++ ) {
-			an[i].an_name.bv_val = attrs[i];
-			an[i].an_name.bv_len = strlen( attrs[i] );
-			an[i].an_desc = NULL;
-			rs.sr_err = slap_bv2ad( &an[i].an_name, &an[i].an_desc, &text );
-			if ( rs.sr_err != LDAP_SUCCESS) {
-				slapi_ch_free( (void **)&an );
-				return -1;
+			an[j].an_name.bv_val = attrs[i];
+			an[j].an_name.bv_len = strlen( attrs[i] );
+			an[j].an_desc = NULL;
+			if ( slap_bv2ad( &an[j].an_name, &an[j].an_desc, &text ) == LDAP_SUCCESS) {
+				j++;
 			}
 		}
-		an[i].an_name.bv_len = 0;
-		an[i].an_name.bv_val = NULL;
+		an[j].an_name.bv_len = 0;
+		an[j].an_name.bv_val = NULL;
 	}
 
 	rs.sr_err = LDAP_SUCCESS;
