@@ -1,8 +1,8 @@
 /* zn_malloc.c - zone-based malloc routines */
-/* $OpenLDAP: pkg/ldap/servers/slapd/zn_malloc.c,v 1.3.2.2 2005/01/20 18:03:56 kurt Exp $*/
+/* $OpenLDAP: pkg/ldap/servers/slapd/zn_malloc.c,v 1.11 2006/02/12 06:03:12 kurt Exp $*/
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2003-2005 The OpenLDAP Foundation.
+ * Copyright 2003-2006 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -13,15 +13,15 @@
  * top-level directory of the distribution or, alternatively, at
  * <http://www.OpenLDAP.org/license.html>.
  */
-/* Copyright 2004 IBM Corporation
+/* Portions Copyright 2004 IBM Corporation
  * All rights reserved.
- * Redisribution and use in source and binary forms, with or without
- * modification, are permitted only as  authorizd by the OpenLADP
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
  * Public License.
  */
 /* ACKNOWLEDGEMENTS
- * This work originally developed by Jong-Hyuk Choi
- * 2004/12/09   jongchoi@OpenLDAP.org
+ * This work originally developed by Jong-Hyuk Choi for inclusion in
+ * OpenLDAP Software.
  */
 
 #include "portable.h"
@@ -542,7 +542,7 @@ slap_zn_free(void *ptr, void *ctx)
 						Debug(LDAP_DEBUG_ANY, "slap_zn_free: "
 							"free object not found while bit is clear.\n",
 							0, 0, 0);
-						assert(zo);
+						assert(zo != NULL);
 
 					}
 				} else {
@@ -604,7 +604,7 @@ slap_zn_free(void *ptr, void *ctx)
 						Debug(LDAP_DEBUG_ANY, "slap_zn_free: "
 							"free object not found while bit is clear.\n",
 							0, 0, 0 );
-						assert( zo );
+						assert(zo != NULL);
 
 					}
 				} else {
